--------------------------------------------------------------------------------
-- Alloy4PV -- Alloy for Process Verification
-- Copyright (C) 2012, 2014  Yoann Laurent (yoann.laurent@lip6.fr)

-- This program is free software: you can redistribute it and/or
-- modify it under the terms of the GNU General Public License as
-- published by the Free Software Foundation, either version 3 of the
-- License, or (at your option) any later version.

-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-- General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see
-- <http://www.gnu.org/licenses/>.
--------------------------------------------------------------------------------

module process

open semantic
open syntax

fact initTokens {
	Init[  
		Initial -> 1 + RequestedOrder -> 1 , 	
		ActivityEdge -> 0 
	]
}

one sig BankConnector extends Resource {}{}

one sig InfoIPSW6 extends Info {} {
	use_i 		= AcceptPayment -> BankConnector 
	timing_i		= (Action -> 1) ++ (ReceiveOrder -> 1 + FillOrder -> 2 + SendInvoice -> 1 +
						MakePayment -> 1 + AcceptPayment -> 2 + ShipOrder -> 3 +
						CloseOrder -> 1) 
}


one sig RequestedOrder extends ActivityParameterNode {} {
	parameterMultiplicityElement = MultiOneOne
	direction = ParameterDirectionIn	
}

one sig Initial      extends InitialNode {} {} 

one sig ReceiveOrder extends OpaqueAction {}{
	input = InputPinReceiveOrder
	output = none
}

one sig DecisionOrder      extends DecisionNode {} {} 

one sig FillOrder extends OpaqueAction {}{
	input = none
	output = none
}

one sig Fork      extends ForkNode {} {} //6

one sig SendInvoice extends OpaqueAction {}{
	input = none
	output = OutputPinInvoice
}
one sig MakePayment extends OpaqueAction {}{
	input = InputPinInvoice
	output = none
}
one sig AcceptPayment extends OpaqueAction {}{
	input = none
	output = none
}

one sig ShipOrder extends OpaqueAction {}{
	input = none
	output = none
}

one sig Join      extends JoinNode {} {} 

one sig Merge      extends MergeNode {} {} 

one sig CloseOrder extends OpaqueAction {}{
	input = none
	output = none
}

one sig Final       extends ActivityFinalNode {} {} 

one sig OutputPinInvoice extends OutputPin {}{
	multiplicityElement = MultiOneOne
}
one sig InputPinInvoice extends InputPin {}{
	multiplicityElement = MultiOneOne
}
one sig InputPinReceiveOrder extends InputPin {}{ //17
	multiplicityElement = MultiOneOne
}

one sig f1 extends ObjectFlow {} {
	source = RequestedOrder
	target = InputPinReceiveOrder
}
one sig f2 extends ControlFlow {} {
	source = Initial
	target = ReceiveOrder
}
one sig f3 extends ControlFlow {} {
	source = ReceiveOrder
	target = DecisionOrder
}
one sig f4 extends ControlFlow {} {
	source = DecisionOrder
	target = FillOrder
}
one sig f5 extends ControlFlow {} {
	source = FillOrder
	target = Fork
}
one sig f6 extends ControlFlow {} {
	source = Fork
	target = SendInvoice
}
one sig f7 extends ControlFlow {} {
	source = SendInvoice
	target = MakePayment
}
one sig f8 extends ObjectFlow {} {
	source = OutputPinInvoice
	target = InputPinInvoice
}
one sig f9 extends ControlFlow {} {
	source = MakePayment
	target = AcceptPayment
}
one sig f10 extends ControlFlow {} {
	source = AcceptPayment
	target = Join
}
one sig f11 extends ControlFlow {} {
	source 	= ShipOrder 
	target 	= Join
}
one sig f12 extends ControlFlow {} {
	source = Join
	target = Merge
}
one sig f13 extends ControlFlow {} {
	source = DecisionOrder
	target = Merge
}
one sig f14 extends ControlFlow {} {
	source = Merge
	target = CloseOrder
}
one sig f15 extends ControlFlow {} {
	source = CloseOrder
	target = Final
}
one sig f16 extends ControlFlow {} {
	source 	= Fork 
	target 	= ShipOrder
}




