--------------------------------------------------------------------------------
-- Alloy4PV -- Alloy for Process Verification
-- Copyright (C) 2012, 2014  Yoann Laurent (yoann.laurent@lip6.fr)

-- This program is free software: you can redistribute it and/or
-- modify it under the terms of the GNU General Public License as
-- published by the Free Software Foundation, either version 3 of the
-- License, or (at your option) any later version.

-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-- General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see
-- <http://www.gnu.org/licenses/>.
--------------------------------------------------------------------------------

module process

open OrderProcess


/* ------------------------ Control-Flow  ------------------------*/
/** OptionToComplete */
pred OptionToComplete {
	some s : State, f: ActivityFinalNode | s.hasTokens[f]
}

/** ProperCompletion */
pred ProperCompletion {
	some s : State, f: ActivityFinalNode | s.hasTokens[f] and (all o:ExecutableNode | not s.hasTokens[o] )
}


/* ------------------------ Data-Flow  ------------------------*/
/** MissingData */
pred pAll[s:State, node:ActivityNode] {  
	not s.hasTokens[node] and 
	some node.incoming and 
	all n:node.incoming | s.hasOffers[n] 
}
pred pAction[s:State, node:Action] { 
	all n:node.incoming | s.hasOffers[n]
	and
	all n:node.input.incoming | s.getOffers[n] >=n.target.multiplicityElement.lower 
}
pred MissingData {
	all s:State, a:Action | pAll[s,a] and not pAction[s,a] implies 
		some ss:(s+s.^next) | pAction[ss,a]
}

/** RedundantData */
pred RedundantData {
	some s : State, f: ActivityFinalNode | s.hasTokens[f] and (all of:ObjectFlow | not s.hasOffers[of])
}

/* ------------------------ Resources ------------------------*/
/** MissingResources */
pred MissingResources {
	all s:State, a:Action, r:Resource | r in Use[a] and  s.isReadyToStart[a] implies {
		all other:(Action-a) | r in Use[other] implies not s.hasTokens[other] 
	}
}


//-- display an execution which terminates
run {OptionToComplete} for 0 but 27 State, 3 Int

check {OptionToComplete} for 0 but 27 State, 3 Int expect 0
check {ProperCompletion} for 0 but 27 State, 3 Int expect 0
check {MissingData} for 0 but 27 State, 3 Int expect 0
check {RedundantData} for 0 but 27 State, 3 Int expect 0
check {MissingResources} for 0 but 27 State, 3 Int expect 0


/** *Visualization Variables */
// http://alloy.mit.edu/community/node/548
fun vNodeExecuting : State->ActivityNode {
   {s:State, a:ActivityNode | s.hasTokens[a] }
}
fun vEdgeHaveOffers : State->ActivityEdge {
   {s:State, e:ActivityEdge | s.hasOffers[e] }
}



