--------------------------------------------------------------------------------
-- Alloy4PV -- Alloy for Process Verification
-- Copyright (C) 2012, 2014  Yoann Laurent (yoann.laurent@lip6.fr)

-- This program is free software: you can redistribute it and/or
-- modify it under the terms of the GNU General Public License as
-- published by the Free Software Foundation, either version 3 of the
-- License, or (at your option) any later version.

-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-- General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see
-- <http://www.gnu.org/licenses/>.
--------------------------------------------------------------------------------

module syntax

open util/integer
open util/boolean

abstract sig Object {}

/************************************************************************
fUML.Syntax.Classes.Kernel
************************************************************************/
abstract sig RedefinableElement extends NamedElement {}
abstract sig Element extends Object {} 
abstract sig NamedElement extends 	Element {}
abstract sig Class extends BehavioredClassifier {}
abstract sig Classifier extends Type {}
abstract sig Type extends Namespace {}
abstract sig Namespace extends PackageableElement {}
abstract sig PackageableElement extends NamedElement {}
abstract sig MultiplicityElement extends Element {
	upper : Int,
	lower : Int
}
abstract sig TypedElement extends NamedElement {}
abstract sig Parameter extends TypedElement {}

/************************************************************************
fUML.Syntax.Activities.IntermediateActivities
************************************************************************/
abstract sig ActivityEdge extends RedefinableElement {
	source : ActivityNode,
	target : ActivityNode,
}
abstract sig ActivityNode extends RedefinableElement {
	outgoing : set ActivityEdge,
	incoming : set ActivityEdge
}
abstract sig ControlNode extends ActivityNode {}
abstract sig FinalNode extends ControlNode {}
abstract sig ActivityFinalNode extends FinalNode {}
abstract sig FlowFinalNode extends FinalNode {}
abstract sig Activity extends Behavior {}
abstract sig ActivityParameterNode extends ObjectNode {
	parameterMultiplicityElement : MultiplicityElement,
	direction : ParameterDirectionKind
}
abstract sig ParameterDirectionKind extends Element {}
abstract sig ObjectNode extends ActivityNode {}
abstract sig ControlFlow extends ActivityEdge {}
abstract sig ObjectFlow extends ActivityEdge {}
abstract sig DecisionNode extends ControlNode {}
abstract sig MergeNode extends ControlNode {}
abstract sig ForkNode extends ControlNode {}
abstract sig JoinNode extends ControlNode {}
abstract sig InitialNode extends ControlNode {}

/************************************************************************
fUML.Syntax.CommonBehaviors.BasicBehaviors
************************************************************************/
abstract sig Behavior extends Class {}
abstract sig BehavioredClassifier extends Classifier {}

/************************************************************************
fUML.Syntax.Activities.ExtraStructuredActivities
************************************************************************/
abstract sig ExpansionKind extends Object {}
abstract sig EK_parallel, EK_iterative, EK_stream extends ExpansionKind {}
abstract sig ExpansionNode extends ObjectNode {}
abstract sig ExpansionRegion extends StructuredActivityNode {}

/************************************************************************
fUML.Syntax.Activities.CompleteStructuredActivities
************************************************************************/
abstract sig Clause extends Element {}
abstract sig ConditionalNode extends StructuredActivityNode {}
abstract sig ExecutableNode extends ActivityNode {}
abstract sig LoopNode extends StructuredActivityNode {}
abstract sig StructuredActivityNode extends Action {}

/************************************************************************
fUML.Syntax.Actions.BasicActions
************************************************************************/
abstract sig Action extends ExecutableNode {
	output : set OutputPin,
	input : set InputPin
} 
abstract sig CallAction extends InvocationAction {}
abstract sig CallBehaviorAction extends CallAction {}
abstract sig CallOperationAction extends CallAction {}
abstract sig SendSignalAction extends InvocationAction {}
abstract sig OpaqueAction extends Action {}
abstract sig InputPin extends Pin {}
abstract sig OutputPin extends Pin {}
abstract sig Pin extends ObjectNode {
	multiplicityElement : MultiplicityElement
}
abstract sig InvocationAction extends Action {}

/************************************************************************
sig - help to write simple process
************************************************************************/

one sig ParameterDirectionIn extends ParameterDirectionKind {}
one sig ParameterDirectionOut extends ParameterDirectionKind {}

one sig MultiZeroOne extends MultiplicityElement {} {
	lower = 0
	upper = 1
}
one sig MultiOneOne extends MultiplicityElement {} {
	lower = 1
	upper = 1
}
one sig MultiZeroStar extends MultiplicityElement {} {
	lower = 0
	upper = max[] 
}
one sig MultiOneStar extends MultiplicityElement {} {
	lower = 1
	upper = max[] 
}
one sig MultiTwoStar extends MultiplicityElement {} {
	lower = 2
	upper = max[] 
}
one sig MultiStarStar extends MultiplicityElement {} {
	lower = max[]
	upper = max[] 
}

fact nodeAndEdgeConnectivity {
	all node : ActivityNode, edge : ActivityEdge | edge.source = node    implies edge in node.outgoing
	and 
	all node : ActivityNode, edge : ActivityEdge | edge.target  = node    implies edge in node.incoming
	and
	all node : ActivityNode, edge : ActivityEdge | edge in node.outgoing implies edge.source = node
	and
	all node : ActivityNode, edge : ActivityEdge | edge in node.incoming  implies edge.target     = node
}








